(herald "IKEv1 Public-Key" (algebra diffie-hellman) (bound 22)
  (limit 8000))

(comment "CPSA 3.4.0")
(comment "All input read from IKEv1_pk2_a.scm")
(comment "Step count limited to 8000")
(comment "Strand count bounded at 22")

(defprotocol pk2-a diffie-hellman
  (defrole init
    (vars (ni nr ci cr lst algo text) (i expn) (Gr expr) (I R name))
    (trace
      (send
        (cat I R ci lst (enc ni (pubk R))
          (enc (exp (gen) i) (hash "prf" ni ci))
          (enc I (hash "prf" ni ci))))
      (recv
        (cat R I ci cr algo (enc nr (pubk I))
          (enc (exp (gen) Gr) (hash "prf" nr cr))
          (enc R (hash "prf" nr cr))
          (hash ni nr (exp (gen) (mul i Gr)) (exp (gen) i)
            (exp (gen) Gr) cr ci lst R)))
      (send
        (cat I R ci cr
          (hash ni nr (exp (gen) (mul i Gr)) (exp (gen) i)
            (exp (gen) Gr) ci cr lst I))))
    (uniq-orig ni ci)
    (uniq-gen i)
    (neq ((exp (gen) Gr) (gen))))
  (defrole resp
    (vars (ni nr ci cr lst algo text) (r expn) (Gi expr) (I R name))
    (trace
      (recv
        (cat I R ci lst (enc ni (pubk R))
          (enc (exp (gen) Gi) (hash "prf" ni ci))
          (enc I (hash "prf" ni ci))))
      (send
        (cat R I ci cr algo (enc nr (pubk I))
          (enc (exp (gen) r) (hash "prf" nr cr))
          (enc R (hash "prf" nr cr))
          (hash ni nr (exp (gen) (mul r Gi)) (exp (gen) Gi)
            (exp (gen) r) cr ci lst R)))
      (recv
        (cat I R ci cr
          (hash ni nr (exp (gen) (mul r Gi)) (exp (gen) Gi)
            (exp (gen) r) ci cr lst I))) (send "blab"))
    (uniq-orig nr cr)
    (uniq-gen r)
    (neq ((exp (gen) Gi) (gen)))
    (absent (r (exp (gen) Gi))))
  (defrole blabber-akey
    (vars (key akey))
    (trace (recv "blab") (send key))
    (uniq-orig key)))

(defskeleton pk2-a
  (vars (ni nr ci cr lst algo text) (I R name) (i expn) (Gr expr))
  (defstrand blabber-akey 2 (key (privk R)))
  (defstrand init 3 (ni ni) (nr nr) (ci ci) (cr cr) (lst lst)
    (algo algo) (I I) (R R) (i i) (Gr Gr))
  (precedes ((1 2) (0 0)))
  (neq ((exp (gen) Gr) (gen)))
  (pen-non-orig (privk R))
  (uniq-gen i)
  (uniq-orig ni ci (privk R))
  (goals
    (forall
      ((iend brend brstart node) (I R name) (ni nr ci cr lst algo text)
        (i expn) (Gr base))
      (implies
        (and (p "init" 2 iend) (p "init" "R" iend R)
          (p "init" "I" iend I) (p "init" "ni" iend ni)
          (p "init" "nr" iend nr) (p "init" "ci" iend ci)
          (p "init" "cr" iend cr) (p "init" "lst" iend lst)
          (p "init" "i" iend i) (p "init" "Gr" iend Gr)
          (p "init" "algo" iend algo) (p "blabber-akey" 1 brend)
          (str-prec brstart brend) (p "blabber-akey" 0 brstart)
          (p "blabber-akey" "key" brend (privk R)) (prec iend brstart)
          (pnon (privk R)))
        (or
          (exists ((r1 node)) (and (p "resp" 0 r1) (p "resp" "R" r1 R)))
          (exists ((i1 node))
            (and (p "init" 0 i1) (p "init" "I" i1 R))))))
    (forall
      ((iend brend brstart node) (I R name) (ni nr ci cr lst algo text)
        (i expn) (Gr base))
      (implies
        (and (p "init" 2 iend) (p "init" "R" iend R)
          (p "init" "I" iend I) (p "init" "ni" iend ni)
          (p "init" "nr" iend nr) (p "init" "ci" iend ci)
          (p "init" "cr" iend cr) (p "init" "lst" iend lst)
          (p "init" "i" iend i) (p "init" "Gr" iend Gr)
          (p "init" "algo" iend algo) (p "blabber-akey" 1 brend)
          (str-prec brstart brend) (p "blabber-akey" 0 brstart)
          (p "blabber-akey" "key" brend (privk R)) (prec iend brstart)
          (pnon (privk R)))
        (or
          (exists ((r1 node))
            (and (p "resp" 0 r1) (p "resp" "R" r1 R)
              (p "resp" "I" r1 I)))
          (exists ((i1 node))
            (and (p "init" 0 i1) (p "init" "I" i1 R)
              (p "init" "R" i1 I))))))
    (forall
      ((iend brend brstart node) (I R name) (ni nr ci cr lst algo text)
        (i expn) (Gr base))
      (implies
        (and (p "init" 2 iend) (p "init" "R" iend R)
          (p "init" "I" iend I) (p "init" "ni" iend ni)
          (p "init" "nr" iend nr) (p "init" "ci" iend ci)
          (p "init" "cr" iend cr) (p "init" "lst" iend lst)
          (p "init" "i" iend i) (p "init" "Gr" iend Gr)
          (p "init" "algo" iend algo) (p "blabber-akey" 1 brend)
          (str-prec brstart brend) (p "blabber-akey" 0 brstart)
          (p "blabber-akey" "key" brend (privk R)) (prec iend brstart)
          (pnon (privk R)))
        (exists ((r1 node) (Gi base) (r expn))
          (and (p "resp" 0 r1) (p "resp" "R" r1 R) (p "resp" "I" r1 I)
            (p "resp" "Gi" r1 Gi) (p "resp" "r" r1 r)
            (= (exp Gi r) (exp Gr i))))))
    (forall
      ((iend brend brstart node) (I R name) (ni nr ci cr lst algo text)
        (i expn) (Gr base))
      (implies
        (and (p "init" 2 iend) (p "init" "R" iend R)
          (p "init" "I" iend I) (p "init" "ni" iend ni)
          (p "init" "nr" iend nr) (p "init" "ci" iend ci)
          (p "init" "cr" iend cr) (p "init" "lst" iend lst)
          (p "init" "i" iend i) (p "init" "Gr" iend Gr)
          (p "init" "algo" iend algo) (p "blabber-akey" 1 brend)
          (str-prec brstart brend) (p "blabber-akey" 0 brstart)
          (p "blabber-akey" "key" brend (privk R)) (prec iend brstart)
          (pnon (privk R)))
        (exists ((r1 node) (r expn))
          (and (p "resp" 0 r1) (p "resp" "R" r1 R) (p "resp" "I" r1 I)
            (p "resp" "ni" r1 ni) (p "resp" "nr" r1 nr)
            (p "resp" "r" r1 r) (p "resp" "ci" r1 ci)
            (p "resp" "cr" r1 cr) (p "resp" "lst" r1 lst)
            (p "resp" "Gi" r1 (exp (gen) i))
            (p "resp" "algo" r1 algo))))))
  (traces ((recv "blab") (send (privk R)))
    ((send
       (cat I R ci lst (enc ni (pubk R))
         (enc (exp (gen) i) (hash "prf" ni ci))
         (enc I (hash "prf" ni ci))))
      (recv
        (cat R I ci cr algo (enc nr (pubk I))
          (enc (exp (gen) Gr) (hash "prf" nr cr))
          (enc R (hash "prf" nr cr))
          (hash ni nr (exp (gen) (mul i Gr)) (exp (gen) i)
            (exp (gen) Gr) cr ci lst R)))
      (send
        (cat I R ci cr
          (hash ni nr (exp (gen) (mul i Gr)) (exp (gen) i)
            (exp (gen) Gr) ci cr lst I))))))

(defprotocol pk2-a diffie-hellman
  (defrole init
    (vars (ni nr ci cr lst algo text) (i expn) (Gr expr) (I R name))
    (trace
      (send
        (cat I R ci lst (enc ni (pubk R))
          (enc (exp (gen) i) (hash "prf" ni ci))
          (enc I (hash "prf" ni ci))))
      (recv
        (cat R I ci cr algo (enc nr (pubk I))
          (enc (exp (gen) Gr) (hash "prf" nr cr))
          (enc R (hash "prf" nr cr))
          (hash ni nr (exp (gen) (mul i Gr)) (exp (gen) i)
            (exp (gen) Gr) cr ci lst R)))
      (send
        (cat I R ci cr
          (hash ni nr (exp (gen) (mul i Gr)) (exp (gen) i)
            (exp (gen) Gr) ci cr lst I))))
    (uniq-orig ni ci)
    (uniq-gen i)
    (neq ((exp (gen) Gr) (gen))))
  (defrole resp
    (vars (ni nr ci cr lst algo text) (r expn) (Gi expr) (I R name))
    (trace
      (recv
        (cat I R ci lst (enc ni (pubk R))
          (enc (exp (gen) Gi) (hash "prf" ni ci))
          (enc I (hash "prf" ni ci))))
      (send
        (cat R I ci cr algo (enc nr (pubk I))
          (enc (exp (gen) r) (hash "prf" nr cr))
          (enc R (hash "prf" nr cr))
          (hash ni nr (exp (gen) (mul r Gi)) (exp (gen) Gi)
            (exp (gen) r) cr ci lst R)))
      (recv
        (cat I R ci cr
          (hash ni nr (exp (gen) (mul r Gi)) (exp (gen) Gi)
            (exp (gen) r) ci cr lst I))) (send "blab"))
    (uniq-orig nr cr)
    (uniq-gen r)
    (neq ((exp (gen) Gi) (gen)))
    (absent (r (exp (gen) Gi))))
  (defrole blabber-akey
    (vars (key akey))
    (trace (recv "blab") (send key))
    (uniq-orig key)))

(defskeleton pk2-a
  (vars (ni nr ci cr lst algo text) (I R name) (r expn) (Gi expr))
  (defstrand blabber-akey 2 (key (privk I)))
  (defstrand resp 4 (ni ni) (nr nr) (ci ci) (cr cr) (lst lst)
    (algo algo) (I I) (R R) (r r) (Gi Gi))
  (precedes ((1 3) (0 0)))
  (absent (r (exp (gen) Gi)))
  (neq ((exp (gen) Gi) (gen)))
  (pen-non-orig (privk I))
  (uniq-gen r)
  (uniq-orig nr cr (privk I))
  (goals
    (forall
      ((rend biend bistart node) (I R name) (ni nr ci cr lst algo text)
        (r expn) (Gi base))
      (implies
        (and (p "resp" 3 rend) (p "resp" "R" rend R)
          (p "resp" "I" rend I) (p "resp" "ni" rend ni)
          (p "resp" "nr" rend nr) (p "resp" "ci" rend ci)
          (p "resp" "cr" rend cr) (p "resp" "lst" rend lst)
          (p "resp" "algo" rend algo) (p "resp" "r" rend r)
          (p "resp" "Gi" rend Gi) (p "blabber-akey" 1 biend)
          (str-prec bistart biend) (p "blabber-akey" 0 bistart)
          (p "blabber-akey" "key" biend (privk I)) (prec rend bistart)
          (pnon (privk I)))
        (or
          (exists ((r1 node)) (and (p "resp" 0 r1) (p "resp" "R" r1 I)))
          (exists ((i1 node))
            (and (p "init" 0 i1) (p "init" "I" i1 I))))))
    (forall
      ((rend biend bistart node) (I R name) (ni nr ci cr lst algo text)
        (r expn) (Gi base))
      (implies
        (and (p "resp" 3 rend) (p "resp" "R" rend R)
          (p "resp" "I" rend I) (p "resp" "ni" rend ni)
          (p "resp" "nr" rend nr) (p "resp" "ci" rend ci)
          (p "resp" "cr" rend cr) (p "resp" "lst" rend lst)
          (p "resp" "algo" rend algo) (p "resp" "r" rend r)
          (p "resp" "Gi" rend Gi) (p "blabber-akey" 1 biend)
          (str-prec bistart biend) (p "blabber-akey" 0 bistart)
          (p "blabber-akey" "key" biend (privk I)) (prec rend bistart)
          (pnon (privk I)))
        (or
          (exists ((r1 node))
            (and (p "resp" 0 r1) (p "resp" "R" r1 I)
              (p "resp" "I" r1 R)))
          (exists ((i1 node))
            (and (p "init" 0 i1) (p "init" "I" i1 I)
              (p "init" "R" i1 R))))))
    (forall
      ((rend biend bistart node) (I R name) (ni nr ci cr lst algo text)
        (r expn) (Gi base))
      (implies
        (and (p "resp" 3 rend) (p "resp" "R" rend R)
          (p "resp" "I" rend I) (p "resp" "ni" rend ni)
          (p "resp" "nr" rend nr) (p "resp" "ci" rend ci)
          (p "resp" "cr" rend cr) (p "resp" "lst" rend lst)
          (p "resp" "algo" rend algo) (p "resp" "r" rend r)
          (p "resp" "Gi" rend Gi) (p "blabber-akey" 1 biend)
          (str-prec bistart biend) (p "blabber-akey" 0 bistart)
          (p "blabber-akey" "key" biend (privk I)) (prec rend bistart)
          (pnon (privk I)))
        (exists ((i1 node) (Gr base) (i expn))
          (and (p "init" 0 i1) (p "init" "R" i1 R) (p "init" "I" i1 I)
            (p "init" "Gr" i1 Gr) (p "init" "i" i1 i)
            (= (exp Gi r) (exp Gr i))))))
    (forall
      ((rend biend bistart node) (I R name) (ni nr ci cr lst algo text)
        (r expn) (Gi base))
      (implies
        (and (p "resp" 3 rend) (p "resp" "R" rend R)
          (p "resp" "I" rend I) (p "resp" "ni" rend ni)
          (p "resp" "nr" rend nr) (p "resp" "ci" rend ci)
          (p "resp" "cr" rend cr) (p "resp" "lst" rend lst)
          (p "resp" "algo" rend algo) (p "resp" "r" rend r)
          (p "resp" "Gi" rend Gi) (p "blabber-akey" 1 biend)
          (str-prec bistart biend) (p "blabber-akey" 0 bistart)
          (p "blabber-akey" "key" biend (privk I)) (prec rend bistart)
          (pnon (privk I)))
        (exists ((i1 node) (i expn))
          (and (p "init" 0 i1) (p "init" "R" i1 R) (p "init" "I" i1 I)
            (p "init" "ni" i1 ni) (p "init" "nr" i1 nr)
            (p "init" "ci" i1 ci) (p "init" "cr" i1 cr)
            (p "init" "algo" i1 algo) (p "init" "lst" i1 lst)
            (p "init" "Gr" i1 (exp (gen) r)) (p "init" "i" i1 i))))))
  (comment
    "Responder point of view; agreement / weak agreement / aliveness goals")
  (traces ((recv "blab") (send (privk I)))
    ((recv
       (cat I R ci lst (enc ni (pubk R))
         (enc (exp (gen) Gi) (hash "prf" ni ci))
         (enc I (hash "prf" ni ci))))
      (send
        (cat R I ci cr algo (enc nr (pubk I))
          (enc (exp (gen) r) (hash "prf" nr cr))
          (enc R (hash "prf" nr cr))
          (hash ni nr (exp (gen) (mul r Gi)) (exp (gen) Gi)
            (exp (gen) r) cr ci lst R)))
      (recv
        (cat I R ci cr
          (hash ni nr (exp (gen) (mul r Gi)) (exp (gen) Gi)
            (exp (gen) r) ci cr lst I))) (send "blab"))))

(defprotocol pk2-a diffie-hellman
  (defrole init
    (vars (ni nr ci cr lst algo text) (i expn) (Gr expr) (I R name))
    (trace
      (send
        (cat I R ci lst (enc ni (pubk R))
          (enc (exp (gen) i) (hash "prf" ni ci))
          (enc I (hash "prf" ni ci))))
      (recv
        (cat R I ci cr algo (enc nr (pubk I))
          (enc (exp (gen) Gr) (hash "prf" nr cr))
          (enc R (hash "prf" nr cr))
          (hash ni nr (exp (gen) (mul i Gr)) (exp (gen) i)
            (exp (gen) Gr) cr ci lst R)))
      (send
        (cat I R ci cr
          (hash ni nr (exp (gen) (mul i Gr)) (exp (gen) i)
            (exp (gen) Gr) ci cr lst I))))
    (uniq-orig ni ci)
    (uniq-gen i)
    (neq ((exp (gen) Gr) (gen))))
  (defrole resp
    (vars (ni nr ci cr lst algo text) (r expn) (Gi expr) (I R name))
    (trace
      (recv
        (cat I R ci lst (enc ni (pubk R))
          (enc (exp (gen) Gi) (hash "prf" ni ci))
          (enc I (hash "prf" ni ci))))
      (send
        (cat R I ci cr algo (enc nr (pubk I))
          (enc (exp (gen) r) (hash "prf" nr cr))
          (enc R (hash "prf" nr cr))
          (hash ni nr (exp (gen) (mul r Gi)) (exp (gen) Gi)
            (exp (gen) r) cr ci lst R)))
      (recv
        (cat I R ci cr
          (hash ni nr (exp (gen) (mul r Gi)) (exp (gen) Gi)
            (exp (gen) r) ci cr lst I))) (send "blab"))
    (uniq-orig nr cr)
    (uniq-gen r)
    (neq ((exp (gen) Gi) (gen)))
    (absent (r (exp (gen) Gi))))
  (defrole blabber-akey
    (vars (key akey))
    (trace (recv "blab") (send key))
    (uniq-orig key)))

(defskeleton pk2-a
  (vars (ni nr ci cr lst algo text) (I R name) (i expn) (Gr expr))
  (defstrand init 3 (ni ni) (nr nr) (ci ci) (cr cr) (lst lst)
    (algo algo) (I I) (R R) (i i) (Gr Gr))
  (deflistener (hash "KDF" ni nr (exp (gen) (mul i Gr)) ci cr))
  (neq ((exp (gen) Gr) (gen)))
  (non-orig (privk I) (privk R))
  (uniq-gen i)
  (uniq-orig ni ci)
  (traces
    ((send
       (cat I R ci lst (enc ni (pubk R))
         (enc (exp (gen) i) (hash "prf" ni ci))
         (enc I (hash "prf" ni ci))))
      (recv
        (cat R I ci cr algo (enc nr (pubk I))
          (enc (exp (gen) Gr) (hash "prf" nr cr))
          (enc R (hash "prf" nr cr))
          (hash ni nr (exp (gen) (mul i Gr)) (exp (gen) i)
            (exp (gen) Gr) cr ci lst R)))
      (send
        (cat I R ci cr
          (hash ni nr (exp (gen) (mul i Gr)) (exp (gen) i)
            (exp (gen) Gr) ci cr lst I))))
    ((recv (hash "KDF" ni nr (exp (gen) (mul i Gr)) ci cr))
      (send (hash "KDF" ni nr (exp (gen) (mul i Gr)) ci cr)))))

(defprotocol pk2-a diffie-hellman
  (defrole init
    (vars (ni nr ci cr lst algo text) (i expn) (Gr expr) (I R name))
    (trace
      (send
        (cat I R ci lst (enc ni (pubk R))
          (enc (exp (gen) i) (hash "prf" ni ci))
          (enc I (hash "prf" ni ci))))
      (recv
        (cat R I ci cr algo (enc nr (pubk I))
          (enc (exp (gen) Gr) (hash "prf" nr cr))
          (enc R (hash "prf" nr cr))
          (hash ni nr (exp (gen) (mul i Gr)) (exp (gen) i)
            (exp (gen) Gr) cr ci lst R)))
      (send
        (cat I R ci cr
          (hash ni nr (exp (gen) (mul i Gr)) (exp (gen) i)
            (exp (gen) Gr) ci cr lst I))))
    (uniq-orig ni ci)
    (uniq-gen i)
    (neq ((exp (gen) Gr) (gen))))
  (defrole resp
    (vars (ni nr ci cr lst algo text) (r expn) (Gi expr) (I R name))
    (trace
      (recv
        (cat I R ci lst (enc ni (pubk R))
          (enc (exp (gen) Gi) (hash "prf" ni ci))
          (enc I (hash "prf" ni ci))))
      (send
        (cat R I ci cr algo (enc nr (pubk I))
          (enc (exp (gen) r) (hash "prf" nr cr))
          (enc R (hash "prf" nr cr))
          (hash ni nr (exp (gen) (mul r Gi)) (exp (gen) Gi)
            (exp (gen) r) cr ci lst R)))
      (recv
        (cat I R ci cr
          (hash ni nr (exp (gen) (mul r Gi)) (exp (gen) Gi)
            (exp (gen) r) ci cr lst I))) (send "blab"))
    (uniq-orig nr cr)
    (uniq-gen r)
    (neq ((exp (gen) Gi) (gen)))
    (absent (r (exp (gen) Gi))))
  (defrole blabber-akey
    (vars (key akey))
    (trace (recv "blab") (send key))
    (uniq-orig key)))

(defskeleton pk2-a
  (vars (ni nr ci cr lst algo text) (I R name) (r expn) (Gi expr))
  (defstrand resp 3 (ni ni) (nr nr) (ci ci) (cr cr) (lst lst)
    (algo algo) (I I) (R R) (r r) (Gi Gi))
  (deflistener (hash "KDF" ni nr (exp (gen) (mul r Gi)) ci cr))
  (absent (r (exp (gen) Gi)))
  (neq ((exp (gen) Gi) (gen)))
  (non-orig (privk I) (privk R))
  (uniq-gen r)
  (uniq-orig nr cr)
  (traces
    ((recv
       (cat I R ci lst (enc ni (pubk R))
         (enc (exp (gen) Gi) (hash "prf" ni ci))
         (enc I (hash "prf" ni ci))))
      (send
        (cat R I ci cr algo (enc nr (pubk I))
          (enc (exp (gen) r) (hash "prf" nr cr))
          (enc R (hash "prf" nr cr))
          (hash ni nr (exp (gen) (mul r Gi)) (exp (gen) Gi)
            (exp (gen) r) cr ci lst R)))
      (recv
        (cat I R ci cr
          (hash ni nr (exp (gen) (mul r Gi)) (exp (gen) Gi)
            (exp (gen) r) ci cr lst I))))
    ((recv (hash "KDF" ni nr (exp (gen) (mul r Gi)) ci cr))
      (send (hash "KDF" ni nr (exp (gen) (mul r Gi)) ci cr)))))

(defprotocol pk2-a diffie-hellman
  (defrole init
    (vars (ni nr ci cr lst algo text) (i expn) (Gr expr) (I R name))
    (trace
      (send
        (cat I R ci lst (enc ni (pubk R))
          (enc (exp (gen) i) (hash "prf" ni ci))
          (enc I (hash "prf" ni ci))))
      (recv
        (cat R I ci cr algo (enc nr (pubk I))
          (enc (exp (gen) Gr) (hash "prf" nr cr))
          (enc R (hash "prf" nr cr))
          (hash ni nr (exp (gen) (mul i Gr)) (exp (gen) i)
            (exp (gen) Gr) cr ci lst R)))
      (send
        (cat I R ci cr
          (hash ni nr (exp (gen) (mul i Gr)) (exp (gen) i)
            (exp (gen) Gr) ci cr lst I))))
    (uniq-orig ni ci)
    (uniq-gen i)
    (neq ((exp (gen) Gr) (gen))))
  (defrole resp
    (vars (ni nr ci cr lst algo text) (r expn) (Gi expr) (I R name))
    (trace
      (recv
        (cat I R ci lst (enc ni (pubk R))
          (enc (exp (gen) Gi) (hash "prf" ni ci))
          (enc I (hash "prf" ni ci))))
      (send
        (cat R I ci cr algo (enc nr (pubk I))
          (enc (exp (gen) r) (hash "prf" nr cr))
          (enc R (hash "prf" nr cr))
          (hash ni nr (exp (gen) (mul r Gi)) (exp (gen) Gi)
            (exp (gen) r) cr ci lst R)))
      (recv
        (cat I R ci cr
          (hash ni nr (exp (gen) (mul r Gi)) (exp (gen) Gi)
            (exp (gen) r) ci cr lst I))) (send "blab"))
    (uniq-orig nr cr)
    (uniq-gen r)
    (neq ((exp (gen) Gi) (gen)))
    (absent (r (exp (gen) Gi))))
  (defrole blabber-akey
    (vars (key akey))
    (trace (recv "blab") (send key))
    (uniq-orig key)))

(defskeleton pk2-a
  (vars (ci lst algo ni nr cr text) (I R name) (i r expn))
  (defstrand resp 4 (ni ni) (nr nr) (ci ci) (cr cr) (lst lst)
    (algo algo) (I I) (R R) (r r) (Gi i))
  (defstrand init 3 (ni ni) (nr nr) (ci ci) (cr cr) (lst lst)
    (algo algo) (I I) (R R) (i i) (Gr r))
  (defstrand blabber-akey 2 (key (privk I)))
  (defstrand blabber-akey 2 (key (privk R)))
  (deflistener (hash "KDF" ni nr (exp (gen) (mul i r)) ci cr))
  (precedes ((0 1) (1 1)) ((0 3) (2 0)) ((0 3) (3 0)) ((1 0) (0 0))
    ((1 2) (0 2)))
  (absent (r (exp (gen) i)))
  (ind-zero-in (r (exp (gen) i)))
  (neq ((exp (gen) r) (gen)) ((exp (gen) i) (gen)))
  (pen-non-orig (privk I) (privk R))
  (uniq-gen i r)
  (uniq-orig ci ni nr cr (privk I) (privk R))
  (traces
    ((recv
       (cat I R ci lst (enc ni (pubk R))
         (enc (exp (gen) i) (hash "prf" ni ci))
         (enc I (hash "prf" ni ci))))
      (send
        (cat R I ci cr algo (enc nr (pubk I))
          (enc (exp (gen) r) (hash "prf" nr cr))
          (enc R (hash "prf" nr cr))
          (hash ni nr (exp (gen) (mul i r)) (exp (gen) i) (exp (gen) r)
            cr ci lst R)))
      (recv
        (cat I R ci cr
          (hash ni nr (exp (gen) (mul i r)) (exp (gen) i) (exp (gen) r)
            ci cr lst I))) (send "blab"))
    ((send
       (cat I R ci lst (enc ni (pubk R))
         (enc (exp (gen) i) (hash "prf" ni ci))
         (enc I (hash "prf" ni ci))))
      (recv
        (cat R I ci cr algo (enc nr (pubk I))
          (enc (exp (gen) r) (hash "prf" nr cr))
          (enc R (hash "prf" nr cr))
          (hash ni nr (exp (gen) (mul i r)) (exp (gen) i) (exp (gen) r)
            cr ci lst R)))
      (send
        (cat I R ci cr
          (hash ni nr (exp (gen) (mul i r)) (exp (gen) i) (exp (gen) r)
            ci cr lst I)))) ((recv "blab") (send (privk I)))
    ((recv "blab") (send (privk R)))
    ((recv (hash "KDF" ni nr (exp (gen) (mul i r)) ci cr))
      (send (hash "KDF" ni nr (exp (gen) (mul i r)) ci cr)))))

(defprotocol pk2-a diffie-hellman
  (defrole init
    (vars (ni nr ci cr lst algo text) (i expn) (Gr expr) (I R name))
    (trace
      (send
        (cat I R ci lst (enc ni (pubk R))
          (enc (exp (gen) i) (hash "prf" ni ci))
          (enc I (hash "prf" ni ci))))
      (recv
        (cat R I ci cr algo (enc nr (pubk I))
          (enc (exp (gen) Gr) (hash "prf" nr cr))
          (enc R (hash "prf" nr cr))
          (hash ni nr (exp (gen) (mul i Gr)) (exp (gen) i)
            (exp (gen) Gr) cr ci lst R)))
      (send
        (cat I R ci cr
          (hash ni nr (exp (gen) (mul i Gr)) (exp (gen) i)
            (exp (gen) Gr) ci cr lst I))))
    (uniq-orig ni ci)
    (uniq-gen i)
    (neq ((exp (gen) Gr) (gen))))
  (defrole resp
    (vars (ni nr ci cr lst algo text) (r expn) (Gi expr) (I R name))
    (trace
      (recv
        (cat I R ci lst (enc ni (pubk R))
          (enc (exp (gen) Gi) (hash "prf" ni ci))
          (enc I (hash "prf" ni ci))))
      (send
        (cat R I ci cr algo (enc nr (pubk I))
          (enc (exp (gen) r) (hash "prf" nr cr))
          (enc R (hash "prf" nr cr))
          (hash ni nr (exp (gen) (mul r Gi)) (exp (gen) Gi)
            (exp (gen) r) cr ci lst R)))
      (recv
        (cat I R ci cr
          (hash ni nr (exp (gen) (mul r Gi)) (exp (gen) Gi)
            (exp (gen) r) ci cr lst I))) (send "blab"))
    (uniq-orig nr cr)
    (uniq-gen r)
    (neq ((exp (gen) Gi) (gen)))
    (absent (r (exp (gen) Gi))))
  (defrole blabber-akey
    (vars (key akey))
    (trace (recv "blab") (send key))
    (uniq-orig key)))

(defskeleton pk2-a
  (vars (ni nr ci cr lst algo text) (I R name) (i expn) (Gr expr))
  (defstrand init 3 (ni ni) (nr nr) (ci ci) (cr cr) (lst lst)
    (algo algo) (I I) (R R) (i i) (Gr Gr))
  (defstrand blabber-akey 2 (key (privk I)))
  (defstrand blabber-akey 2 (key (privk R)))
  (deflistener (hash "KDF" ni nr (exp (gen) (mul i Gr)) ci cr))
  (precedes ((0 2) (1 0)) ((0 2) (2 0)))
  (neq ((exp (gen) Gr) (gen)))
  (pen-non-orig (privk I) (privk R))
  (uniq-gen i)
  (uniq-orig ni ci (privk I) (privk R))
  (traces
    ((send
       (cat I R ci lst (enc ni (pubk R))
         (enc (exp (gen) i) (hash "prf" ni ci))
         (enc I (hash "prf" ni ci))))
      (recv
        (cat R I ci cr algo (enc nr (pubk I))
          (enc (exp (gen) Gr) (hash "prf" nr cr))
          (enc R (hash "prf" nr cr))
          (hash ni nr (exp (gen) (mul i Gr)) (exp (gen) i)
            (exp (gen) Gr) cr ci lst R)))
      (send
        (cat I R ci cr
          (hash ni nr (exp (gen) (mul i Gr)) (exp (gen) i)
            (exp (gen) Gr) ci cr lst I))))
    ((recv "blab") (send (privk I))) ((recv "blab") (send (privk R)))
    ((recv (hash "KDF" ni nr (exp (gen) (mul i Gr)) ci cr))
      (send (hash "KDF" ni nr (exp (gen) (mul i Gr)) ci cr)))))

(defprotocol pk2-a diffie-hellman
  (defrole init
    (vars (ni nr ci cr lst algo text) (i expn) (Gr expr) (I R name))
    (trace
      (send
        (cat I R ci lst (enc ni (pubk R))
          (enc (exp (gen) i) (hash "prf" ni ci))
          (enc I (hash "prf" ni ci))))
      (recv
        (cat R I ci cr algo (enc nr (pubk I))
          (enc (exp (gen) Gr) (hash "prf" nr cr))
          (enc R (hash "prf" nr cr))
          (hash ni nr (exp (gen) (mul i Gr)) (exp (gen) i)
            (exp (gen) Gr) cr ci lst R)))
      (send
        (cat I R ci cr
          (hash ni nr (exp (gen) (mul i Gr)) (exp (gen) i)
            (exp (gen) Gr) ci cr lst I))))
    (uniq-orig ni ci)
    (uniq-gen i)
    (neq ((exp (gen) Gr) (gen))))
  (defrole resp
    (vars (ni nr ci cr lst algo text) (r expn) (Gi expr) (I R name))
    (trace
      (recv
        (cat I R ci lst (enc ni (pubk R))
          (enc (exp (gen) Gi) (hash "prf" ni ci))
          (enc I (hash "prf" ni ci))))
      (send
        (cat R I ci cr algo (enc nr (pubk I))
          (enc (exp (gen) r) (hash "prf" nr cr))
          (enc R (hash "prf" nr cr))
          (hash ni nr (exp (gen) (mul r Gi)) (exp (gen) Gi)
            (exp (gen) r) cr ci lst R)))
      (recv
        (cat I R ci cr
          (hash ni nr (exp (gen) (mul r Gi)) (exp (gen) Gi)
            (exp (gen) r) ci cr lst I))) (send "blab"))
    (uniq-orig nr cr)
    (uniq-gen r)
    (neq ((exp (gen) Gi) (gen)))
    (absent (r (exp (gen) Gi))))
  (defrole blabber-akey
    (vars (key akey))
    (trace (recv "blab") (send key))
    (uniq-orig key)))

(defskeleton pk2-a
  (vars (ni nr ci cr lst algo text) (I R name) (r expn) (Gi expr))
  (defstrand resp 4 (ni ni) (nr nr) (ci ci) (cr cr) (lst lst)
    (algo algo) (I I) (R R) (r r) (Gi Gi))
  (defstrand blabber-akey 2 (key (privk I)))
  (defstrand blabber-akey 2 (key (privk R)))
  (deflistener (hash "KDF" ni nr (exp (gen) (mul r Gi)) ci cr))
  (precedes ((0 3) (1 0)) ((0 3) (2 0)))
  (absent (r (exp (gen) Gi)))
  (neq ((exp (gen) Gi) (gen)))
  (pen-non-orig (privk I) (privk R))
  (uniq-gen r)
  (uniq-orig nr cr (privk I) (privk R))
  (traces
    ((recv
       (cat I R ci lst (enc ni (pubk R))
         (enc (exp (gen) Gi) (hash "prf" ni ci))
         (enc I (hash "prf" ni ci))))
      (send
        (cat R I ci cr algo (enc nr (pubk I))
          (enc (exp (gen) r) (hash "prf" nr cr))
          (enc R (hash "prf" nr cr))
          (hash ni nr (exp (gen) (mul r Gi)) (exp (gen) Gi)
            (exp (gen) r) cr ci lst R)))
      (recv
        (cat I R ci cr
          (hash ni nr (exp (gen) (mul r Gi)) (exp (gen) Gi)
            (exp (gen) r) ci cr lst I))) (send "blab"))
    ((recv "blab") (send (privk I))) ((recv "blab") (send (privk R)))
    ((recv (hash "KDF" ni nr (exp (gen) (mul r Gi)) ci cr))
      (send (hash "KDF" ni nr (exp (gen) (mul r Gi)) ci cr)))))
