(herald "IKEv1 Quick Mode" (algebra diffie-hellman) (bound 15))

(comment "CPSA 3.4.0")
(comment "All input read from IKEv1_quick_noid.scm")
(comment "Strand count bounded at 15")

(defprotocol quick diffie-hellman
  (defrole init
    (vars (ni nr mid lst algo text) (i expn) (Gr expr) (I R name))
    (trace
      (send
        (cat I R mid
          (enc (hash "prf" (ltk I R) mid lst ni (exp (gen) i)) lst ni
            (exp (gen) i) (ltk I R))
          (enc (hash "prf" (ltk R I) mid lst ni (exp (gen) i)) lst ni
            (exp (gen) i) (ltk R I))))
      (recv
        (cat R I mid
          (enc (hash "prf" (ltk I R) mid ni algo nr (exp (gen) Gr)) algo
            nr (exp (gen) Gr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni algo nr (exp (gen) Gr)) algo
            nr (exp (gen) Gr) (ltk R I))))
      (send
        (cat I R mid (enc (hash "prf" (ltk I R) mid ni nr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni nr) (ltk R I)))))
    (uniq-orig ni mid)
    (uniq-gen i))
  (defrole resp
    (vars (ni nr mid lst algo text) (r expn) (Gi expr) (I R name))
    (trace
      (recv
        (cat I R mid
          (enc (hash "prf" (ltk I R) mid lst ni (exp (gen) Gi)) lst ni
            (exp (gen) Gi) (ltk I R))
          (enc (hash "prf" (ltk R I) mid lst ni (exp (gen) Gi)) lst ni
            (exp (gen) Gi) (ltk R I))))
      (send
        (cat R I mid
          (enc (hash "prf" (ltk I R) mid ni algo nr (exp (gen) r)) algo
            nr (exp (gen) r) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni algo nr (exp (gen) r)) algo
            nr (exp (gen) r) (ltk R I))))
      (recv
        (cat I R mid (enc (hash "prf" (ltk I R) mid ni nr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni nr) (ltk R I))))
      (send "blab"))
    (uniq-orig nr)
    (uniq-gen r)
    (absent (r (exp (gen) Gi))))
  (defrole blabber-skey
    (vars (key skey))
    (trace (recv "blab") (send key))
    (uniq-orig key)))

(defskeleton quick
  (vars (ni nr mid lst algo text) (I R name) (i expn) (Gr expr))
  (defstrand init 3 (ni ni) (nr nr) (mid mid) (lst lst) (algo algo)
    (I I) (R R) (i i) (Gr Gr))
  (non-orig (ltk I R) (ltk R I))
  (uniq-gen i)
  (uniq-orig ni mid)
  (goals
    (forall
      ((iend node) (I R name) (ni nr mid lst algo text) (i expn)
        (Gr base))
      (implies
        (and (p "init" 2 iend) (p "init" "R" iend R)
          (p "init" "I" iend I) (p "init" "ni" iend ni)
          (p "init" "nr" iend nr) (p "init" "mid" iend mid)
          (p "init" "lst" iend lst) (p "init" "i" iend i)
          (p "init" "Gr" iend Gr) (p "init" "algo" iend algo)
          (non (ltk R I)) (non (ltk I R)))
        (or
          (exists ((r1 node)) (and (p "resp" 0 r1) (p "resp" "R" r1 R)))
          (exists ((i1 node))
            (and (p "init" 0 i1) (p "init" "I" i1 R))))))
    (forall
      ((iend node) (I R name) (ni nr mid lst algo text) (i expn)
        (Gr base))
      (implies
        (and (p "init" 2 iend) (p "init" "R" iend R)
          (p "init" "I" iend I) (p "init" "ni" iend ni)
          (p "init" "nr" iend nr) (p "init" "mid" iend mid)
          (p "init" "lst" iend lst) (p "init" "i" iend i)
          (p "init" "Gr" iend Gr) (p "init" "algo" iend algo)
          (non (ltk R I)) (non (ltk I R)))
        (or
          (exists ((r1 node))
            (and (p "resp" 0 r1) (p "resp" "R" r1 R)
              (p "resp" "I" r1 I)))
          (exists ((i1 node))
            (and (p "init" 0 i1) (p "init" "I" i1 R)
              (p "init" "R" i1 I))))))
    (forall
      ((iend node) (I R name) (ni nr mid lst algo text) (i expn)
        (Gr base))
      (implies
        (and (p "init" 2 iend) (p "init" "R" iend R)
          (p "init" "I" iend I) (p "init" "ni" iend ni)
          (p "init" "nr" iend nr) (p "init" "mid" iend mid)
          (p "init" "lst" iend lst) (p "init" "i" iend i)
          (p "init" "Gr" iend Gr) (p "init" "algo" iend algo)
          (non (ltk R I)) (non (ltk I R)))
        (exists ((r1 node) (Gi base) (r expn))
          (and (p "resp" 0 r1) (p "resp" "R" r1 R) (p "resp" "I" r1 I)
            (p "resp" "Gi" r1 Gi) (p "resp" "r" r1 r)
            (= (exp Gi r) (exp Gr i))))))
    (forall
      ((iend node) (I R name) (ni nr mid lst algo text) (i expn)
        (Gr base))
      (implies
        (and (p "init" 2 iend) (p "init" "R" iend R)
          (p "init" "I" iend I) (p "init" "ni" iend ni)
          (p "init" "nr" iend nr) (p "init" "mid" iend mid)
          (p "init" "lst" iend lst) (p "init" "i" iend i)
          (p "init" "Gr" iend Gr) (p "init" "algo" iend algo)
          (non (ltk R I)) (non (ltk I R)))
        (exists ((r1 node) (r expn))
          (and (p "resp" 0 r1) (p "resp" "R" r1 R) (p "resp" "I" r1 I)
            (p "resp" "ni" r1 ni) (p "resp" "nr" r1 nr)
            (p "resp" "r" r1 r) (p "resp" "mid" r1 mid)
            (p "resp" "lst" r1 lst) (p "resp" "Gi" r1 (exp (gen) i))
            (p "resp" "algo" r1 algo))))))
  (traces
    ((send
       (cat I R mid
         (enc (hash "prf" (ltk I R) mid lst ni (exp (gen) i)) lst ni
           (exp (gen) i) (ltk I R))
         (enc (hash "prf" (ltk R I) mid lst ni (exp (gen) i)) lst ni
           (exp (gen) i) (ltk R I))))
      (recv
        (cat R I mid
          (enc (hash "prf" (ltk I R) mid ni algo nr (exp (gen) Gr)) algo
            nr (exp (gen) Gr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni algo nr (exp (gen) Gr)) algo
            nr (exp (gen) Gr) (ltk R I))))
      (send
        (cat I R mid (enc (hash "prf" (ltk I R) mid ni nr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni nr) (ltk R I)))))))

(defprotocol quick diffie-hellman
  (defrole init
    (vars (ni nr mid lst algo text) (i expn) (Gr expr) (I R name))
    (trace
      (send
        (cat I R mid
          (enc (hash "prf" (ltk I R) mid lst ni (exp (gen) i)) lst ni
            (exp (gen) i) (ltk I R))
          (enc (hash "prf" (ltk R I) mid lst ni (exp (gen) i)) lst ni
            (exp (gen) i) (ltk R I))))
      (recv
        (cat R I mid
          (enc (hash "prf" (ltk I R) mid ni algo nr (exp (gen) Gr)) algo
            nr (exp (gen) Gr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni algo nr (exp (gen) Gr)) algo
            nr (exp (gen) Gr) (ltk R I))))
      (send
        (cat I R mid (enc (hash "prf" (ltk I R) mid ni nr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni nr) (ltk R I)))))
    (uniq-orig ni mid)
    (uniq-gen i))
  (defrole resp
    (vars (ni nr mid lst algo text) (r expn) (Gi expr) (I R name))
    (trace
      (recv
        (cat I R mid
          (enc (hash "prf" (ltk I R) mid lst ni (exp (gen) Gi)) lst ni
            (exp (gen) Gi) (ltk I R))
          (enc (hash "prf" (ltk R I) mid lst ni (exp (gen) Gi)) lst ni
            (exp (gen) Gi) (ltk R I))))
      (send
        (cat R I mid
          (enc (hash "prf" (ltk I R) mid ni algo nr (exp (gen) r)) algo
            nr (exp (gen) r) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni algo nr (exp (gen) r)) algo
            nr (exp (gen) r) (ltk R I))))
      (recv
        (cat I R mid (enc (hash "prf" (ltk I R) mid ni nr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni nr) (ltk R I))))
      (send "blab"))
    (uniq-orig nr)
    (uniq-gen r)
    (absent (r (exp (gen) Gi))))
  (defrole blabber-skey
    (vars (key skey))
    (trace (recv "blab") (send key))
    (uniq-orig key)))

(defskeleton quick
  (vars (ni nr mid lst algo text) (I R name) (r expn) (Gi expr))
  (defstrand resp 4 (ni ni) (nr nr) (mid mid) (lst lst) (algo algo)
    (I I) (R R) (r r) (Gi Gi))
  (absent (r (exp (gen) Gi)))
  (non-orig (ltk I R) (ltk R I))
  (uniq-gen r)
  (uniq-orig nr)
  (goals
    (forall
      ((rend node) (I R name) (ni nr mid lst algo text) (r expn)
        (Gi base))
      (implies
        (and (p "resp" 3 rend) (p "resp" "R" rend R)
          (p "resp" "I" rend I) (p "resp" "ni" rend ni)
          (p "resp" "nr" rend nr) (p "resp" "mid" rend mid)
          (p "resp" "lst" rend lst) (p "resp" "algo" rend algo)
          (p "resp" "r" rend r) (p "resp" "Gi" rend Gi) (non (ltk R I))
          (non (ltk I R)))
        (or
          (exists ((r1 node)) (and (p "resp" 0 r1) (p "resp" "R" r1 I)))
          (exists ((i1 node))
            (and (p "init" 0 i1) (p "init" "I" i1 I))))))
    (forall
      ((rend node) (I R name) (ni nr mid lst algo text) (r expn)
        (Gi base))
      (implies
        (and (p "resp" 3 rend) (p "resp" "R" rend R)
          (p "resp" "I" rend I) (p "resp" "ni" rend ni)
          (p "resp" "nr" rend nr) (p "resp" "mid" rend mid)
          (p "resp" "lst" rend lst) (p "resp" "algo" rend algo)
          (p "resp" "r" rend r) (p "resp" "Gi" rend Gi) (non (ltk R I))
          (non (ltk I R)))
        (or
          (exists ((r1 node))
            (and (p "resp" 0 r1) (p "resp" "R" r1 I)
              (p "resp" "I" r1 R)))
          (exists ((i1 node))
            (and (p "init" 0 i1) (p "init" "I" i1 I)
              (p "init" "R" i1 R))))))
    (forall
      ((rend node) (I R name) (ni nr mid lst algo text) (r expn)
        (Gi base))
      (implies
        (and (p "resp" 3 rend) (p "resp" "R" rend R)
          (p "resp" "I" rend I) (p "resp" "ni" rend ni)
          (p "resp" "nr" rend nr) (p "resp" "mid" rend mid)
          (p "resp" "lst" rend lst) (p "resp" "algo" rend algo)
          (p "resp" "r" rend r) (p "resp" "Gi" rend Gi) (non (ltk R I))
          (non (ltk I R)))
        (exists ((i1 node) (Gr base) (i expn))
          (and (p "init" 0 i1) (p "init" "R" i1 R) (p "init" "I" i1 I)
            (p "init" "Gr" i1 Gr) (p "init" "i" i1 i)
            (= (exp Gi r) (exp Gr i))))))
    (forall
      ((rend node) (I R name) (ni nr mid lst algo text) (r expn)
        (Gi base))
      (implies
        (and (p "resp" 3 rend) (p "resp" "R" rend R)
          (p "resp" "I" rend I) (p "resp" "ni" rend ni)
          (p "resp" "nr" rend nr) (p "resp" "mid" rend mid)
          (p "resp" "lst" rend lst) (p "resp" "algo" rend algo)
          (p "resp" "r" rend r) (p "resp" "Gi" rend Gi) (non (ltk R I))
          (non (ltk I R)))
        (exists ((i1 node) (i expn))
          (and (p "init" 0 i1) (p "init" "R" i1 R) (p "init" "I" i1 I)
            (p "init" "ni" i1 ni) (p "init" "nr" i1 nr)
            (p "init" "mid" i1 mid) (p "init" "algo" i1 algo)
            (p "init" "lst" i1 lst) (p "init" "Gr" i1 (exp (gen) r))
            (p "init" "i" i1 i))))))
  (comment
    "Responder point of view; agreement / weak agreement / aliveness goals")
  (traces
    ((recv
       (cat I R mid
         (enc (hash "prf" (ltk I R) mid lst ni (exp (gen) Gi)) lst ni
           (exp (gen) Gi) (ltk I R))
         (enc (hash "prf" (ltk R I) mid lst ni (exp (gen) Gi)) lst ni
           (exp (gen) Gi) (ltk R I))))
      (send
        (cat R I mid
          (enc (hash "prf" (ltk I R) mid ni algo nr (exp (gen) r)) algo
            nr (exp (gen) r) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni algo nr (exp (gen) r)) algo
            nr (exp (gen) r) (ltk R I))))
      (recv
        (cat I R mid (enc (hash "prf" (ltk I R) mid ni nr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni nr) (ltk R I))))
      (send "blab"))))

(defprotocol quick diffie-hellman
  (defrole init
    (vars (ni nr mid lst algo text) (i expn) (Gr expr) (I R name))
    (trace
      (send
        (cat I R mid
          (enc (hash "prf" (ltk I R) mid lst ni (exp (gen) i)) lst ni
            (exp (gen) i) (ltk I R))
          (enc (hash "prf" (ltk R I) mid lst ni (exp (gen) i)) lst ni
            (exp (gen) i) (ltk R I))))
      (recv
        (cat R I mid
          (enc (hash "prf" (ltk I R) mid ni algo nr (exp (gen) Gr)) algo
            nr (exp (gen) Gr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni algo nr (exp (gen) Gr)) algo
            nr (exp (gen) Gr) (ltk R I))))
      (send
        (cat I R mid (enc (hash "prf" (ltk I R) mid ni nr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni nr) (ltk R I)))))
    (uniq-orig ni mid)
    (uniq-gen i))
  (defrole resp
    (vars (ni nr mid lst algo text) (r expn) (Gi expr) (I R name))
    (trace
      (recv
        (cat I R mid
          (enc (hash "prf" (ltk I R) mid lst ni (exp (gen) Gi)) lst ni
            (exp (gen) Gi) (ltk I R))
          (enc (hash "prf" (ltk R I) mid lst ni (exp (gen) Gi)) lst ni
            (exp (gen) Gi) (ltk R I))))
      (send
        (cat R I mid
          (enc (hash "prf" (ltk I R) mid ni algo nr (exp (gen) r)) algo
            nr (exp (gen) r) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni algo nr (exp (gen) r)) algo
            nr (exp (gen) r) (ltk R I))))
      (recv
        (cat I R mid (enc (hash "prf" (ltk I R) mid ni nr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni nr) (ltk R I))))
      (send "blab"))
    (uniq-orig nr)
    (uniq-gen r)
    (absent (r (exp (gen) Gi))))
  (defrole blabber-skey
    (vars (key skey))
    (trace (recv "blab") (send key))
    (uniq-orig key)))

(defskeleton quick
  (vars (ni nr lst algo mid lst-0 algo-0 text) (I R name) (i expn)
    (Gr expr))
  (defstrand init 3 (ni ni) (nr nr) (mid mid) (lst lst-0) (algo algo-0)
    (I I) (R R) (i i) (Gr Gr))
  (deflistener
    (hash "prf" (ltk I R) (exp (gen) (mul i Gr)) lst algo ni nr))
  (non-orig (ltk I R) (ltk R I))
  (uniq-gen i)
  (uniq-orig ni mid)
  (traces
    ((send
       (cat I R mid
         (enc (hash "prf" (ltk I R) mid lst-0 ni (exp (gen) i)) lst-0 ni
           (exp (gen) i) (ltk I R))
         (enc (hash "prf" (ltk R I) mid lst-0 ni (exp (gen) i)) lst-0 ni
           (exp (gen) i) (ltk R I))))
      (recv
        (cat R I mid
          (enc (hash "prf" (ltk I R) mid ni algo-0 nr (exp (gen) Gr))
            algo-0 nr (exp (gen) Gr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni algo-0 nr (exp (gen) Gr))
            algo-0 nr (exp (gen) Gr) (ltk R I))))
      (send
        (cat I R mid (enc (hash "prf" (ltk I R) mid ni nr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni nr) (ltk R I)))))
    ((recv (hash "prf" (ltk I R) (exp (gen) (mul i Gr)) lst algo ni nr))
      (send
        (hash "prf" (ltk I R) (exp (gen) (mul i Gr)) lst algo ni nr)))))

(defprotocol quick diffie-hellman
  (defrole init
    (vars (ni nr mid lst algo text) (i expn) (Gr expr) (I R name))
    (trace
      (send
        (cat I R mid
          (enc (hash "prf" (ltk I R) mid lst ni (exp (gen) i)) lst ni
            (exp (gen) i) (ltk I R))
          (enc (hash "prf" (ltk R I) mid lst ni (exp (gen) i)) lst ni
            (exp (gen) i) (ltk R I))))
      (recv
        (cat R I mid
          (enc (hash "prf" (ltk I R) mid ni algo nr (exp (gen) Gr)) algo
            nr (exp (gen) Gr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni algo nr (exp (gen) Gr)) algo
            nr (exp (gen) Gr) (ltk R I))))
      (send
        (cat I R mid (enc (hash "prf" (ltk I R) mid ni nr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni nr) (ltk R I)))))
    (uniq-orig ni mid)
    (uniq-gen i))
  (defrole resp
    (vars (ni nr mid lst algo text) (r expn) (Gi expr) (I R name))
    (trace
      (recv
        (cat I R mid
          (enc (hash "prf" (ltk I R) mid lst ni (exp (gen) Gi)) lst ni
            (exp (gen) Gi) (ltk I R))
          (enc (hash "prf" (ltk R I) mid lst ni (exp (gen) Gi)) lst ni
            (exp (gen) Gi) (ltk R I))))
      (send
        (cat R I mid
          (enc (hash "prf" (ltk I R) mid ni algo nr (exp (gen) r)) algo
            nr (exp (gen) r) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni algo nr (exp (gen) r)) algo
            nr (exp (gen) r) (ltk R I))))
      (recv
        (cat I R mid (enc (hash "prf" (ltk I R) mid ni nr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni nr) (ltk R I))))
      (send "blab"))
    (uniq-orig nr)
    (uniq-gen r)
    (absent (r (exp (gen) Gi))))
  (defrole blabber-skey
    (vars (key skey))
    (trace (recv "blab") (send key))
    (uniq-orig key)))

(defskeleton quick
  (vars (ni nr lst algo mid lst-0 algo-0 text) (I R name) (i expn)
    (Gr expr))
  (defstrand init 3 (ni ni) (nr nr) (mid mid) (lst lst-0) (algo algo-0)
    (I I) (R R) (i i) (Gr Gr))
  (deflistener
    (hash "prf" (ltk R I) (exp (gen) (mul i Gr)) lst algo ni nr))
  (non-orig (ltk I R) (ltk R I))
  (uniq-gen i)
  (uniq-orig ni mid)
  (traces
    ((send
       (cat I R mid
         (enc (hash "prf" (ltk I R) mid lst-0 ni (exp (gen) i)) lst-0 ni
           (exp (gen) i) (ltk I R))
         (enc (hash "prf" (ltk R I) mid lst-0 ni (exp (gen) i)) lst-0 ni
           (exp (gen) i) (ltk R I))))
      (recv
        (cat R I mid
          (enc (hash "prf" (ltk I R) mid ni algo-0 nr (exp (gen) Gr))
            algo-0 nr (exp (gen) Gr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni algo-0 nr (exp (gen) Gr))
            algo-0 nr (exp (gen) Gr) (ltk R I))))
      (send
        (cat I R mid (enc (hash "prf" (ltk I R) mid ni nr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni nr) (ltk R I)))))
    ((recv (hash "prf" (ltk R I) (exp (gen) (mul i Gr)) lst algo ni nr))
      (send
        (hash "prf" (ltk R I) (exp (gen) (mul i Gr)) lst algo ni nr)))))

(defprotocol quick diffie-hellman
  (defrole init
    (vars (ni nr mid lst algo text) (i expn) (Gr expr) (I R name))
    (trace
      (send
        (cat I R mid
          (enc (hash "prf" (ltk I R) mid lst ni (exp (gen) i)) lst ni
            (exp (gen) i) (ltk I R))
          (enc (hash "prf" (ltk R I) mid lst ni (exp (gen) i)) lst ni
            (exp (gen) i) (ltk R I))))
      (recv
        (cat R I mid
          (enc (hash "prf" (ltk I R) mid ni algo nr (exp (gen) Gr)) algo
            nr (exp (gen) Gr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni algo nr (exp (gen) Gr)) algo
            nr (exp (gen) Gr) (ltk R I))))
      (send
        (cat I R mid (enc (hash "prf" (ltk I R) mid ni nr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni nr) (ltk R I)))))
    (uniq-orig ni mid)
    (uniq-gen i))
  (defrole resp
    (vars (ni nr mid lst algo text) (r expn) (Gi expr) (I R name))
    (trace
      (recv
        (cat I R mid
          (enc (hash "prf" (ltk I R) mid lst ni (exp (gen) Gi)) lst ni
            (exp (gen) Gi) (ltk I R))
          (enc (hash "prf" (ltk R I) mid lst ni (exp (gen) Gi)) lst ni
            (exp (gen) Gi) (ltk R I))))
      (send
        (cat R I mid
          (enc (hash "prf" (ltk I R) mid ni algo nr (exp (gen) r)) algo
            nr (exp (gen) r) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni algo nr (exp (gen) r)) algo
            nr (exp (gen) r) (ltk R I))))
      (recv
        (cat I R mid (enc (hash "prf" (ltk I R) mid ni nr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni nr) (ltk R I))))
      (send "blab"))
    (uniq-orig nr)
    (uniq-gen r)
    (absent (r (exp (gen) Gi))))
  (defrole blabber-skey
    (vars (key skey))
    (trace (recv "blab") (send key))
    (uniq-orig key)))

(defskeleton quick
  (vars (ni nr lst algo mid lst-0 algo-0 text) (I R name) (r expn)
    (Gi expr))
  (defstrand resp 3 (ni ni) (nr nr) (mid mid) (lst lst-0) (algo algo-0)
    (I I) (R R) (r r) (Gi Gi))
  (deflistener
    (hash "prf" (ltk I R) (exp (gen) (mul r Gi)) lst algo ni nr))
  (absent (r (exp (gen) Gi)))
  (non-orig (ltk I R) (ltk R I))
  (uniq-gen r)
  (uniq-orig nr)
  (traces
    ((recv
       (cat I R mid
         (enc (hash "prf" (ltk I R) mid lst-0 ni (exp (gen) Gi)) lst-0
           ni (exp (gen) Gi) (ltk I R))
         (enc (hash "prf" (ltk R I) mid lst-0 ni (exp (gen) Gi)) lst-0
           ni (exp (gen) Gi) (ltk R I))))
      (send
        (cat R I mid
          (enc (hash "prf" (ltk I R) mid ni algo-0 nr (exp (gen) r))
            algo-0 nr (exp (gen) r) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni algo-0 nr (exp (gen) r))
            algo-0 nr (exp (gen) r) (ltk R I))))
      (recv
        (cat I R mid (enc (hash "prf" (ltk I R) mid ni nr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni nr) (ltk R I)))))
    ((recv (hash "prf" (ltk I R) (exp (gen) (mul r Gi)) lst algo ni nr))
      (send
        (hash "prf" (ltk I R) (exp (gen) (mul r Gi)) lst algo ni nr)))))

(defprotocol quick diffie-hellman
  (defrole init
    (vars (ni nr mid lst algo text) (i expn) (Gr expr) (I R name))
    (trace
      (send
        (cat I R mid
          (enc (hash "prf" (ltk I R) mid lst ni (exp (gen) i)) lst ni
            (exp (gen) i) (ltk I R))
          (enc (hash "prf" (ltk R I) mid lst ni (exp (gen) i)) lst ni
            (exp (gen) i) (ltk R I))))
      (recv
        (cat R I mid
          (enc (hash "prf" (ltk I R) mid ni algo nr (exp (gen) Gr)) algo
            nr (exp (gen) Gr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni algo nr (exp (gen) Gr)) algo
            nr (exp (gen) Gr) (ltk R I))))
      (send
        (cat I R mid (enc (hash "prf" (ltk I R) mid ni nr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni nr) (ltk R I)))))
    (uniq-orig ni mid)
    (uniq-gen i))
  (defrole resp
    (vars (ni nr mid lst algo text) (r expn) (Gi expr) (I R name))
    (trace
      (recv
        (cat I R mid
          (enc (hash "prf" (ltk I R) mid lst ni (exp (gen) Gi)) lst ni
            (exp (gen) Gi) (ltk I R))
          (enc (hash "prf" (ltk R I) mid lst ni (exp (gen) Gi)) lst ni
            (exp (gen) Gi) (ltk R I))))
      (send
        (cat R I mid
          (enc (hash "prf" (ltk I R) mid ni algo nr (exp (gen) r)) algo
            nr (exp (gen) r) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni algo nr (exp (gen) r)) algo
            nr (exp (gen) r) (ltk R I))))
      (recv
        (cat I R mid (enc (hash "prf" (ltk I R) mid ni nr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni nr) (ltk R I))))
      (send "blab"))
    (uniq-orig nr)
    (uniq-gen r)
    (absent (r (exp (gen) Gi))))
  (defrole blabber-skey
    (vars (key skey))
    (trace (recv "blab") (send key))
    (uniq-orig key)))

(defskeleton quick
  (vars (ni nr lst algo mid lst-0 algo-0 text) (I R name) (r expn)
    (Gi expr))
  (defstrand resp 3 (ni ni) (nr nr) (mid mid) (lst lst-0) (algo algo-0)
    (I I) (R R) (r r) (Gi Gi))
  (deflistener
    (hash "prf" (ltk R I) (exp (gen) (mul r Gi)) lst algo ni nr))
  (absent (r (exp (gen) Gi)))
  (non-orig (ltk I R) (ltk R I))
  (uniq-gen r)
  (uniq-orig nr)
  (traces
    ((recv
       (cat I R mid
         (enc (hash "prf" (ltk I R) mid lst-0 ni (exp (gen) Gi)) lst-0
           ni (exp (gen) Gi) (ltk I R))
         (enc (hash "prf" (ltk R I) mid lst-0 ni (exp (gen) Gi)) lst-0
           ni (exp (gen) Gi) (ltk R I))))
      (send
        (cat R I mid
          (enc (hash "prf" (ltk I R) mid ni algo-0 nr (exp (gen) r))
            algo-0 nr (exp (gen) r) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni algo-0 nr (exp (gen) r))
            algo-0 nr (exp (gen) r) (ltk R I))))
      (recv
        (cat I R mid (enc (hash "prf" (ltk I R) mid ni nr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni nr) (ltk R I)))))
    ((recv (hash "prf" (ltk R I) (exp (gen) (mul r Gi)) lst algo ni nr))
      (send
        (hash "prf" (ltk R I) (exp (gen) (mul r Gi)) lst algo ni nr)))))

(defprotocol quick diffie-hellman
  (defrole init
    (vars (ni nr mid lst algo text) (i expn) (Gr expr) (I R name))
    (trace
      (send
        (cat I R mid
          (enc (hash "prf" (ltk I R) mid lst ni (exp (gen) i)) lst ni
            (exp (gen) i) (ltk I R))
          (enc (hash "prf" (ltk R I) mid lst ni (exp (gen) i)) lst ni
            (exp (gen) i) (ltk R I))))
      (recv
        (cat R I mid
          (enc (hash "prf" (ltk I R) mid ni algo nr (exp (gen) Gr)) algo
            nr (exp (gen) Gr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni algo nr (exp (gen) Gr)) algo
            nr (exp (gen) Gr) (ltk R I))))
      (send
        (cat I R mid (enc (hash "prf" (ltk I R) mid ni nr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni nr) (ltk R I)))))
    (uniq-orig ni mid)
    (uniq-gen i))
  (defrole resp
    (vars (ni nr mid lst algo text) (r expn) (Gi expr) (I R name))
    (trace
      (recv
        (cat I R mid
          (enc (hash "prf" (ltk I R) mid lst ni (exp (gen) Gi)) lst ni
            (exp (gen) Gi) (ltk I R))
          (enc (hash "prf" (ltk R I) mid lst ni (exp (gen) Gi)) lst ni
            (exp (gen) Gi) (ltk R I))))
      (send
        (cat R I mid
          (enc (hash "prf" (ltk I R) mid ni algo nr (exp (gen) r)) algo
            nr (exp (gen) r) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni algo nr (exp (gen) r)) algo
            nr (exp (gen) r) (ltk R I))))
      (recv
        (cat I R mid (enc (hash "prf" (ltk I R) mid ni nr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni nr) (ltk R I))))
      (send "blab"))
    (uniq-orig nr)
    (uniq-gen r)
    (absent (r (exp (gen) Gi))))
  (defrole blabber-skey
    (vars (key skey))
    (trace (recv "blab") (send key))
    (uniq-orig key)))

(defskeleton quick
  (vars (nr algo ni mid lst text) (I R name) (i r expn))
  (defstrand resp 4 (ni ni) (nr nr) (mid mid) (lst lst) (algo algo)
    (I I) (R R) (r r) (Gi i))
  (defstrand init 3 (ni ni) (nr nr) (mid mid) (lst lst) (algo algo)
    (I I) (R R) (i i) (Gr r))
  (defstrand blabber-skey 2 (key (ltk I R)))
  (defstrand blabber-skey 2 (key (ltk R I)))
  (deflistener
    (hash "prf" (ltk I R) (exp (gen) (mul i r)) lst algo ni nr))
  (precedes ((0 1) (1 1)) ((0 3) (2 0)) ((0 3) (3 0)) ((1 0) (0 0))
    ((1 2) (0 2)))
  (absent (r (exp (gen) i)))
  (ind-zero-in (r (exp (gen) i)))
  (pen-non-orig (ltk I R) (ltk R I))
  (uniq-gen i r)
  (uniq-orig nr ni mid (ltk I R) (ltk R I))
  (traces
    ((recv
       (cat I R mid
         (enc (hash "prf" (ltk I R) mid lst ni (exp (gen) i)) lst ni
           (exp (gen) i) (ltk I R))
         (enc (hash "prf" (ltk R I) mid lst ni (exp (gen) i)) lst ni
           (exp (gen) i) (ltk R I))))
      (send
        (cat R I mid
          (enc (hash "prf" (ltk I R) mid ni algo nr (exp (gen) r)) algo
            nr (exp (gen) r) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni algo nr (exp (gen) r)) algo
            nr (exp (gen) r) (ltk R I))))
      (recv
        (cat I R mid (enc (hash "prf" (ltk I R) mid ni nr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni nr) (ltk R I))))
      (send "blab"))
    ((send
       (cat I R mid
         (enc (hash "prf" (ltk I R) mid lst ni (exp (gen) i)) lst ni
           (exp (gen) i) (ltk I R))
         (enc (hash "prf" (ltk R I) mid lst ni (exp (gen) i)) lst ni
           (exp (gen) i) (ltk R I))))
      (recv
        (cat R I mid
          (enc (hash "prf" (ltk I R) mid ni algo nr (exp (gen) r)) algo
            nr (exp (gen) r) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni algo nr (exp (gen) r)) algo
            nr (exp (gen) r) (ltk R I))))
      (send
        (cat I R mid (enc (hash "prf" (ltk I R) mid ni nr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni nr) (ltk R I)))))
    ((recv "blab") (send (ltk I R))) ((recv "blab") (send (ltk R I)))
    ((recv (hash "prf" (ltk I R) (exp (gen) (mul i r)) lst algo ni nr))
      (send
        (hash "prf" (ltk I R) (exp (gen) (mul i r)) lst algo ni nr)))))

(defprotocol quick diffie-hellman
  (defrole init
    (vars (ni nr mid lst algo text) (i expn) (Gr expr) (I R name))
    (trace
      (send
        (cat I R mid
          (enc (hash "prf" (ltk I R) mid lst ni (exp (gen) i)) lst ni
            (exp (gen) i) (ltk I R))
          (enc (hash "prf" (ltk R I) mid lst ni (exp (gen) i)) lst ni
            (exp (gen) i) (ltk R I))))
      (recv
        (cat R I mid
          (enc (hash "prf" (ltk I R) mid ni algo nr (exp (gen) Gr)) algo
            nr (exp (gen) Gr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni algo nr (exp (gen) Gr)) algo
            nr (exp (gen) Gr) (ltk R I))))
      (send
        (cat I R mid (enc (hash "prf" (ltk I R) mid ni nr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni nr) (ltk R I)))))
    (uniq-orig ni mid)
    (uniq-gen i))
  (defrole resp
    (vars (ni nr mid lst algo text) (r expn) (Gi expr) (I R name))
    (trace
      (recv
        (cat I R mid
          (enc (hash "prf" (ltk I R) mid lst ni (exp (gen) Gi)) lst ni
            (exp (gen) Gi) (ltk I R))
          (enc (hash "prf" (ltk R I) mid lst ni (exp (gen) Gi)) lst ni
            (exp (gen) Gi) (ltk R I))))
      (send
        (cat R I mid
          (enc (hash "prf" (ltk I R) mid ni algo nr (exp (gen) r)) algo
            nr (exp (gen) r) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni algo nr (exp (gen) r)) algo
            nr (exp (gen) r) (ltk R I))))
      (recv
        (cat I R mid (enc (hash "prf" (ltk I R) mid ni nr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni nr) (ltk R I))))
      (send "blab"))
    (uniq-orig nr)
    (uniq-gen r)
    (absent (r (exp (gen) Gi))))
  (defrole blabber-skey
    (vars (key skey))
    (trace (recv "blab") (send key))
    (uniq-orig key)))

(defskeleton quick
  (vars (nr algo ni mid lst text) (I R name) (i r expn))
  (defstrand resp 4 (ni ni) (nr nr) (mid mid) (lst lst) (algo algo)
    (I I) (R R) (r r) (Gi i))
  (defstrand init 3 (ni ni) (nr nr) (mid mid) (lst lst) (algo algo)
    (I I) (R R) (i i) (Gr r))
  (defstrand blabber-skey 2 (key (ltk I R)))
  (defstrand blabber-skey 2 (key (ltk R I)))
  (deflistener
    (hash "prf" (ltk R I) (exp (gen) (mul i r)) lst algo ni nr))
  (precedes ((0 1) (1 1)) ((0 3) (2 0)) ((0 3) (3 0)) ((1 0) (0 0))
    ((1 2) (0 2)))
  (absent (r (exp (gen) i)))
  (ind-zero-in (r (exp (gen) i)))
  (pen-non-orig (ltk I R) (ltk R I))
  (uniq-gen i r)
  (uniq-orig nr ni mid (ltk I R) (ltk R I))
  (traces
    ((recv
       (cat I R mid
         (enc (hash "prf" (ltk I R) mid lst ni (exp (gen) i)) lst ni
           (exp (gen) i) (ltk I R))
         (enc (hash "prf" (ltk R I) mid lst ni (exp (gen) i)) lst ni
           (exp (gen) i) (ltk R I))))
      (send
        (cat R I mid
          (enc (hash "prf" (ltk I R) mid ni algo nr (exp (gen) r)) algo
            nr (exp (gen) r) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni algo nr (exp (gen) r)) algo
            nr (exp (gen) r) (ltk R I))))
      (recv
        (cat I R mid (enc (hash "prf" (ltk I R) mid ni nr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni nr) (ltk R I))))
      (send "blab"))
    ((send
       (cat I R mid
         (enc (hash "prf" (ltk I R) mid lst ni (exp (gen) i)) lst ni
           (exp (gen) i) (ltk I R))
         (enc (hash "prf" (ltk R I) mid lst ni (exp (gen) i)) lst ni
           (exp (gen) i) (ltk R I))))
      (recv
        (cat R I mid
          (enc (hash "prf" (ltk I R) mid ni algo nr (exp (gen) r)) algo
            nr (exp (gen) r) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni algo nr (exp (gen) r)) algo
            nr (exp (gen) r) (ltk R I))))
      (send
        (cat I R mid (enc (hash "prf" (ltk I R) mid ni nr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni nr) (ltk R I)))))
    ((recv "blab") (send (ltk I R))) ((recv "blab") (send (ltk R I)))
    ((recv (hash "prf" (ltk R I) (exp (gen) (mul i r)) lst algo ni nr))
      (send
        (hash "prf" (ltk R I) (exp (gen) (mul i r)) lst algo ni nr)))))

(defprotocol quick diffie-hellman
  (defrole init
    (vars (ni nr mid lst algo text) (i expn) (Gr expr) (I R name))
    (trace
      (send
        (cat I R mid
          (enc (hash "prf" (ltk I R) mid lst ni (exp (gen) i)) lst ni
            (exp (gen) i) (ltk I R))
          (enc (hash "prf" (ltk R I) mid lst ni (exp (gen) i)) lst ni
            (exp (gen) i) (ltk R I))))
      (recv
        (cat R I mid
          (enc (hash "prf" (ltk I R) mid ni algo nr (exp (gen) Gr)) algo
            nr (exp (gen) Gr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni algo nr (exp (gen) Gr)) algo
            nr (exp (gen) Gr) (ltk R I))))
      (send
        (cat I R mid (enc (hash "prf" (ltk I R) mid ni nr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni nr) (ltk R I)))))
    (uniq-orig ni mid)
    (uniq-gen i))
  (defrole resp
    (vars (ni nr mid lst algo text) (r expn) (Gi expr) (I R name))
    (trace
      (recv
        (cat I R mid
          (enc (hash "prf" (ltk I R) mid lst ni (exp (gen) Gi)) lst ni
            (exp (gen) Gi) (ltk I R))
          (enc (hash "prf" (ltk R I) mid lst ni (exp (gen) Gi)) lst ni
            (exp (gen) Gi) (ltk R I))))
      (send
        (cat R I mid
          (enc (hash "prf" (ltk I R) mid ni algo nr (exp (gen) r)) algo
            nr (exp (gen) r) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni algo nr (exp (gen) r)) algo
            nr (exp (gen) r) (ltk R I))))
      (recv
        (cat I R mid (enc (hash "prf" (ltk I R) mid ni nr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni nr) (ltk R I))))
      (send "blab"))
    (uniq-orig nr)
    (uniq-gen r)
    (absent (r (exp (gen) Gi))))
  (defrole blabber-skey
    (vars (key skey))
    (trace (recv "blab") (send key))
    (uniq-orig key)))

(defskeleton quick
  (vars (ni nr lst algo mid lst-0 algo-0 text) (I R name) (i expn)
    (Gr expr))
  (defstrand init 3 (ni ni) (nr nr) (mid mid) (lst lst-0) (algo algo-0)
    (I I) (R R) (i i) (Gr Gr))
  (defstrand blabber-skey 2 (key (ltk I R)))
  (defstrand blabber-skey 2 (key (ltk R I)))
  (deflistener
    (hash "prf" (ltk I R) (exp (gen) (mul i Gr)) lst algo ni nr))
  (precedes ((0 2) (1 0)) ((0 2) (2 0)))
  (pen-non-orig (ltk I R) (ltk R I))
  (uniq-gen i)
  (uniq-orig ni mid (ltk I R) (ltk R I))
  (traces
    ((send
       (cat I R mid
         (enc (hash "prf" (ltk I R) mid lst-0 ni (exp (gen) i)) lst-0 ni
           (exp (gen) i) (ltk I R))
         (enc (hash "prf" (ltk R I) mid lst-0 ni (exp (gen) i)) lst-0 ni
           (exp (gen) i) (ltk R I))))
      (recv
        (cat R I mid
          (enc (hash "prf" (ltk I R) mid ni algo-0 nr (exp (gen) Gr))
            algo-0 nr (exp (gen) Gr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni algo-0 nr (exp (gen) Gr))
            algo-0 nr (exp (gen) Gr) (ltk R I))))
      (send
        (cat I R mid (enc (hash "prf" (ltk I R) mid ni nr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni nr) (ltk R I)))))
    ((recv "blab") (send (ltk I R))) ((recv "blab") (send (ltk R I)))
    ((recv (hash "prf" (ltk I R) (exp (gen) (mul i Gr)) lst algo ni nr))
      (send
        (hash "prf" (ltk I R) (exp (gen) (mul i Gr)) lst algo ni nr)))))

(defprotocol quick diffie-hellman
  (defrole init
    (vars (ni nr mid lst algo text) (i expn) (Gr expr) (I R name))
    (trace
      (send
        (cat I R mid
          (enc (hash "prf" (ltk I R) mid lst ni (exp (gen) i)) lst ni
            (exp (gen) i) (ltk I R))
          (enc (hash "prf" (ltk R I) mid lst ni (exp (gen) i)) lst ni
            (exp (gen) i) (ltk R I))))
      (recv
        (cat R I mid
          (enc (hash "prf" (ltk I R) mid ni algo nr (exp (gen) Gr)) algo
            nr (exp (gen) Gr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni algo nr (exp (gen) Gr)) algo
            nr (exp (gen) Gr) (ltk R I))))
      (send
        (cat I R mid (enc (hash "prf" (ltk I R) mid ni nr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni nr) (ltk R I)))))
    (uniq-orig ni mid)
    (uniq-gen i))
  (defrole resp
    (vars (ni nr mid lst algo text) (r expn) (Gi expr) (I R name))
    (trace
      (recv
        (cat I R mid
          (enc (hash "prf" (ltk I R) mid lst ni (exp (gen) Gi)) lst ni
            (exp (gen) Gi) (ltk I R))
          (enc (hash "prf" (ltk R I) mid lst ni (exp (gen) Gi)) lst ni
            (exp (gen) Gi) (ltk R I))))
      (send
        (cat R I mid
          (enc (hash "prf" (ltk I R) mid ni algo nr (exp (gen) r)) algo
            nr (exp (gen) r) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni algo nr (exp (gen) r)) algo
            nr (exp (gen) r) (ltk R I))))
      (recv
        (cat I R mid (enc (hash "prf" (ltk I R) mid ni nr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni nr) (ltk R I))))
      (send "blab"))
    (uniq-orig nr)
    (uniq-gen r)
    (absent (r (exp (gen) Gi))))
  (defrole blabber-skey
    (vars (key skey))
    (trace (recv "blab") (send key))
    (uniq-orig key)))

(defskeleton quick
  (vars (ni nr lst algo mid lst-0 algo-0 text) (I R name) (i expn)
    (Gr expr))
  (defstrand init 3 (ni ni) (nr nr) (mid mid) (lst lst-0) (algo algo-0)
    (I I) (R R) (i i) (Gr Gr))
  (defstrand blabber-skey 2 (key (ltk I R)))
  (defstrand blabber-skey 2 (key (ltk R I)))
  (deflistener
    (hash "prf" (ltk R I) (exp (gen) (mul i Gr)) lst algo ni nr))
  (precedes ((0 2) (1 0)) ((0 2) (2 0)))
  (pen-non-orig (ltk I R) (ltk R I))
  (uniq-gen i)
  (uniq-orig ni mid (ltk I R) (ltk R I))
  (traces
    ((send
       (cat I R mid
         (enc (hash "prf" (ltk I R) mid lst-0 ni (exp (gen) i)) lst-0 ni
           (exp (gen) i) (ltk I R))
         (enc (hash "prf" (ltk R I) mid lst-0 ni (exp (gen) i)) lst-0 ni
           (exp (gen) i) (ltk R I))))
      (recv
        (cat R I mid
          (enc (hash "prf" (ltk I R) mid ni algo-0 nr (exp (gen) Gr))
            algo-0 nr (exp (gen) Gr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni algo-0 nr (exp (gen) Gr))
            algo-0 nr (exp (gen) Gr) (ltk R I))))
      (send
        (cat I R mid (enc (hash "prf" (ltk I R) mid ni nr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni nr) (ltk R I)))))
    ((recv "blab") (send (ltk I R))) ((recv "blab") (send (ltk R I)))
    ((recv (hash "prf" (ltk R I) (exp (gen) (mul i Gr)) lst algo ni nr))
      (send
        (hash "prf" (ltk R I) (exp (gen) (mul i Gr)) lst algo ni nr)))))

(defprotocol quick diffie-hellman
  (defrole init
    (vars (ni nr mid lst algo text) (i expn) (Gr expr) (I R name))
    (trace
      (send
        (cat I R mid
          (enc (hash "prf" (ltk I R) mid lst ni (exp (gen) i)) lst ni
            (exp (gen) i) (ltk I R))
          (enc (hash "prf" (ltk R I) mid lst ni (exp (gen) i)) lst ni
            (exp (gen) i) (ltk R I))))
      (recv
        (cat R I mid
          (enc (hash "prf" (ltk I R) mid ni algo nr (exp (gen) Gr)) algo
            nr (exp (gen) Gr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni algo nr (exp (gen) Gr)) algo
            nr (exp (gen) Gr) (ltk R I))))
      (send
        (cat I R mid (enc (hash "prf" (ltk I R) mid ni nr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni nr) (ltk R I)))))
    (uniq-orig ni mid)
    (uniq-gen i))
  (defrole resp
    (vars (ni nr mid lst algo text) (r expn) (Gi expr) (I R name))
    (trace
      (recv
        (cat I R mid
          (enc (hash "prf" (ltk I R) mid lst ni (exp (gen) Gi)) lst ni
            (exp (gen) Gi) (ltk I R))
          (enc (hash "prf" (ltk R I) mid lst ni (exp (gen) Gi)) lst ni
            (exp (gen) Gi) (ltk R I))))
      (send
        (cat R I mid
          (enc (hash "prf" (ltk I R) mid ni algo nr (exp (gen) r)) algo
            nr (exp (gen) r) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni algo nr (exp (gen) r)) algo
            nr (exp (gen) r) (ltk R I))))
      (recv
        (cat I R mid (enc (hash "prf" (ltk I R) mid ni nr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni nr) (ltk R I))))
      (send "blab"))
    (uniq-orig nr)
    (uniq-gen r)
    (absent (r (exp (gen) Gi))))
  (defrole blabber-skey
    (vars (key skey))
    (trace (recv "blab") (send key))
    (uniq-orig key)))

(defskeleton quick
  (vars (ni nr lst algo mid lst-0 algo-0 text) (I R name) (r expn)
    (Gi expr))
  (defstrand resp 4 (ni ni) (nr nr) (mid mid) (lst lst-0) (algo algo-0)
    (I I) (R R) (r r) (Gi Gi))
  (defstrand blabber-skey 2 (key (ltk I R)))
  (defstrand blabber-skey 2 (key (ltk R I)))
  (deflistener
    (hash "prf" (ltk I R) (exp (gen) (mul r Gi)) lst algo ni nr))
  (precedes ((0 3) (1 0)) ((0 3) (2 0)))
  (absent (r (exp (gen) Gi)))
  (pen-non-orig (ltk I R) (ltk R I))
  (uniq-gen r)
  (uniq-orig nr (ltk I R) (ltk R I))
  (traces
    ((recv
       (cat I R mid
         (enc (hash "prf" (ltk I R) mid lst-0 ni (exp (gen) Gi)) lst-0
           ni (exp (gen) Gi) (ltk I R))
         (enc (hash "prf" (ltk R I) mid lst-0 ni (exp (gen) Gi)) lst-0
           ni (exp (gen) Gi) (ltk R I))))
      (send
        (cat R I mid
          (enc (hash "prf" (ltk I R) mid ni algo-0 nr (exp (gen) r))
            algo-0 nr (exp (gen) r) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni algo-0 nr (exp (gen) r))
            algo-0 nr (exp (gen) r) (ltk R I))))
      (recv
        (cat I R mid (enc (hash "prf" (ltk I R) mid ni nr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni nr) (ltk R I))))
      (send "blab")) ((recv "blab") (send (ltk I R)))
    ((recv "blab") (send (ltk R I)))
    ((recv (hash "prf" (ltk I R) (exp (gen) (mul r Gi)) lst algo ni nr))
      (send
        (hash "prf" (ltk I R) (exp (gen) (mul r Gi)) lst algo ni nr)))))

(defprotocol quick diffie-hellman
  (defrole init
    (vars (ni nr mid lst algo text) (i expn) (Gr expr) (I R name))
    (trace
      (send
        (cat I R mid
          (enc (hash "prf" (ltk I R) mid lst ni (exp (gen) i)) lst ni
            (exp (gen) i) (ltk I R))
          (enc (hash "prf" (ltk R I) mid lst ni (exp (gen) i)) lst ni
            (exp (gen) i) (ltk R I))))
      (recv
        (cat R I mid
          (enc (hash "prf" (ltk I R) mid ni algo nr (exp (gen) Gr)) algo
            nr (exp (gen) Gr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni algo nr (exp (gen) Gr)) algo
            nr (exp (gen) Gr) (ltk R I))))
      (send
        (cat I R mid (enc (hash "prf" (ltk I R) mid ni nr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni nr) (ltk R I)))))
    (uniq-orig ni mid)
    (uniq-gen i))
  (defrole resp
    (vars (ni nr mid lst algo text) (r expn) (Gi expr) (I R name))
    (trace
      (recv
        (cat I R mid
          (enc (hash "prf" (ltk I R) mid lst ni (exp (gen) Gi)) lst ni
            (exp (gen) Gi) (ltk I R))
          (enc (hash "prf" (ltk R I) mid lst ni (exp (gen) Gi)) lst ni
            (exp (gen) Gi) (ltk R I))))
      (send
        (cat R I mid
          (enc (hash "prf" (ltk I R) mid ni algo nr (exp (gen) r)) algo
            nr (exp (gen) r) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni algo nr (exp (gen) r)) algo
            nr (exp (gen) r) (ltk R I))))
      (recv
        (cat I R mid (enc (hash "prf" (ltk I R) mid ni nr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni nr) (ltk R I))))
      (send "blab"))
    (uniq-orig nr)
    (uniq-gen r)
    (absent (r (exp (gen) Gi))))
  (defrole blabber-skey
    (vars (key skey))
    (trace (recv "blab") (send key))
    (uniq-orig key)))

(defskeleton quick
  (vars (ni nr lst algo mid lst-0 algo-0 text) (I R name) (r expn)
    (Gi expr))
  (defstrand resp 4 (ni ni) (nr nr) (mid mid) (lst lst-0) (algo algo-0)
    (I I) (R R) (r r) (Gi Gi))
  (defstrand blabber-skey 2 (key (ltk I R)))
  (defstrand blabber-skey 2 (key (ltk R I)))
  (deflistener
    (hash "prf" (ltk R I) (exp (gen) (mul r Gi)) lst algo ni nr))
  (precedes ((0 3) (1 0)) ((0 3) (2 0)))
  (absent (r (exp (gen) Gi)))
  (pen-non-orig (ltk I R) (ltk R I))
  (uniq-gen r)
  (uniq-orig nr (ltk I R) (ltk R I))
  (traces
    ((recv
       (cat I R mid
         (enc (hash "prf" (ltk I R) mid lst-0 ni (exp (gen) Gi)) lst-0
           ni (exp (gen) Gi) (ltk I R))
         (enc (hash "prf" (ltk R I) mid lst-0 ni (exp (gen) Gi)) lst-0
           ni (exp (gen) Gi) (ltk R I))))
      (send
        (cat R I mid
          (enc (hash "prf" (ltk I R) mid ni algo-0 nr (exp (gen) r))
            algo-0 nr (exp (gen) r) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni algo-0 nr (exp (gen) r))
            algo-0 nr (exp (gen) r) (ltk R I))))
      (recv
        (cat I R mid (enc (hash "prf" (ltk I R) mid ni nr) (ltk I R))
          (enc (hash "prf" (ltk R I) mid ni nr) (ltk R I))))
      (send "blab")) ((recv "blab") (send (ltk I R)))
    ((recv "blab") (send (ltk R I)))
    ((recv (hash "prf" (ltk R I) (exp (gen) (mul r Gi)) lst algo ni nr))
      (send
        (hash "prf" (ltk R I) (exp (gen) (mul r Gi)) lst algo ni nr)))))
